/**
 *  @file       commands/run.js
 *  @author     Eddie Roosenmaallen <silvermoon82@gmail.com>
 *  @date       November 2022
 *
 *  Fetch and archive the authenticated user's stream
 */

'use strict';

const fs = require('fs');
const path = require('path');
const readline = require('readline');
const { MastoBot } = require('mastobot');

const {
  loadConfig,
  loadConfigFile,
} = require('../lib/util');


exports.command = 'run [directory]';

exports.describe = 'Download and save any toots older or newer than already saved';

exports.builder = {
  'directory': {
    describe: 'The directory in which to store the configuration and data',
    default: '.',
    type: 'path',
  },
  'verbose': {
    describe: 'Be more chatty about what\'s being done',
    default: false,
  },
  'quiet': {
    describe: 'Be less chatty about what\'s happening',
    default: false,
  },
};


exports.handler = async function run$$handler(argv) {
  const config = loadConfig(argv);
  const directory = path.dirname(config._path);
  
  const { api_domain, api_token } = config;
  const api_url = new URL('https://' + config.api_domain).href;

  const mb = new MastoBot(api_url, { api_token });
  const me = (await mb.checkCredentials()).data;


  console.info(`Connected as @${me.acct}@${api_domain}`);

  let page = await mb.getAccountStatuses(me.id);

  let { newest_id, oldest_id } = config;

  let pages = 0;

  do {
    const newest = page.data[0];
    const [oldest] = page.data.slice(-1);

    if (argv.verbose)
    {
      console.log(`045: got ${page.data.length} toots`, page.link);
      console.log('     Newest:', { id: newest.id, created_at: newest.created_at });
      console.log('     Oldest:', { id: oldest.id, created_at: oldest.created_at });
    }

    const toots = page.data;

    toots.forEach(toot => {
      const d = new Date(toot.created_at);
      const subdir = path.join('' + d.getFullYear(), (d.getMonth() + 1).toString(10).padStart(2, '0'));
      const tootDir = path.resolve(directory, subdir);
      const tootFileName = toot.id + '.json';
      const tootFile = path.resolve(tootDir, tootFileName);

      if (toot.account.id === me.id)
        toot.account = me.url;

      if (toot.id > config.newest)
        config.newest = toot.id;
      if (toot.id < config.oldest)
        config.oldest = toot.id;

      if (!fs.existsSync(tootDir))
        fs.mkdirSync(tootDir, { recursive: true });

      if (!fs.existsSync(tootFile))
      {
        fs.writeFileSync(tootFile, JSON.stringify(toot, null, 2));
        argv.verbose && console.info(`  - wrote ${path.join(subdir, tootFileName)}`);
      }
      else
      {
        argv.verbose && console.info(`  - skip ${path.join(subdir, tootFileName)}; exists`);
      }
    });
  }
  while (page.link.next && (page = await mb.get(page.link.next)) && page.data.length);
};

