/**
 *  @file       lib/util.js
 *  @author     Eddie Roosenmaallen <silvermoon82@gmail.com>
 *  @date       November 2022
 *
 *  This module provides some uitility functions
 */

'use strict';

const fs = require('fs');
const path = require('path');

const exitCodes = {
  EXIT_OK:            0,  // no error
  EXIT_ERROR:         1,  // Error, not otherwise specified
  EXIT_ERROR_CONFIG:  2,  // Failed to load local config
};

const defaultConfig = {
  _path: null,
  api_domain: null,
  api_token: null,
};

/**
 *  Copy nonnull enumerable own properties from `sources` onto `dest`
 *
 *  @param  {Object}    dest    Destination object. Will be mutated.
 *  @param  {...Object} sources Source objects. Properties will be assigned
 *    left-first, so in the case of duplicate property names, the
 *    last-listed value will win
 *
 *  @return {Object}            dest, with new properties
 */
function objectAssignNonnull(dest, ...sources)
{
  sources.forEach(src => {
    for (let [p, v] of Object.entries(src))
    {
      if (typeof v === 'undefined' || v === null)
        continue;
      dest[p] = v;
    }
  });

  return dest;
}


/**
 *  Copy nonnull enumerable own properties from `sources` onto `dest`,
 *  without clobbering existing properties.
 *
 *  @param  {Object}    dest    Destination object. Will be mutated.
 *  @param  {...Object} sources Source objects. Properties will be assigned
 *    left-first, so in the case of duplicate property names, the
 *    first-listed value will win
 *
 *  @return {Object}            dest, with new properties
 */
function objectAssignNoclobber(dest, ...sources)
{
  sources.forEach(src => {
    for (let [p, v] of Object.entries(src))
    {
      if (typeof v === 'undefined' || v === null)
        continue;
      if (typeof dest[p] !== 'undefined')
        continue;
      dest[p] = v;
    }
  });

  return dest;
}


/**
 *  Handle a fatal error processing a command
 *
 *  @param  {Error} error Error caught by cli framework
 */
function handleCommandError(msg, error, yargs)
{
  const code = '' + (error.code || null);
  console.log({msg, error});

  if (code.startsWith('ESYNTAX.'))
  {
    console.error(`Fatal error: ${error.message}
Filename: ${error.fileName}
Position: ${error.column}`);
    process.exit(EXIT_ERROR_CONFIG);
  }

  console.error(`Unexpected ${error.name}:`, error.message);
  console.error(error.stack);
  process.exit(EXIT_UNKNOWN);
}


/**
 *  Load a specified configuration file
 *
 *  @param  {String} filePath Path to the config file to open
 *  @return {Object}          Configuration parsed from the file
 */
function loadConfigFile(filePath = './masto-dl.conf')
{
  filePath = path.resolve(filePath);
  const config = Object.assign({}, defaultConfig);

  // If local config doesn't exist, return empty object
  if (!fs.existsSync(filePath))
    return config;

  try
  {
    const fileConfig = JSON.parse(fs.readFileSync(filePath, 'utf8'));
    objectAssignNonnull(config, fileConfig);
    config._path = filePath;
  }
  catch (error)
  {
    const msg = '' + error.message;

    error.message = 'Error loading configuration: ' + error.message;
    error.fileName = filePath;

    if (error instanceof SyntaxError)
    {
      const [, column] = msg.match(/on line ([0-9]+)$/);

      if (column)
        error.column = column;
    }
    // else if (other predictable error) {} ...
    else
    {
      console.error(`Unexpected ${error.name} when loading configuration: ${error.message}`, '\n' + error.stack);
    }
  }

  return config;
}


/**
 *  Load a local configuration, given a parsed options object
 *
 *  @param  {object} argv CLI Options from yargs
 *  @return {Object}      Configuration from loadConfigFile
 */
function loadConfig(argv, defaultsFromConfigFile = true)
{
  const basePath = path.resolve(argv.directory);

  if (!fs.existsSync(basePath))
  {
    console.error(`Error: Configuration path "${basePath}" does not exist`);
    process.exit(exitCodes.EXIT_CONFIG_NOTDIR);
  }

  const configFile = path.resolve(basePath, argv.configFile);
  const config = loadConfigFile(configFile);

  return config;
}

function saveConfigFile(config, filePath = './masto-dl.conf')
{
  filePath = path.resolve(filePath);

  const configToWrite = objectAssignNonnull(Object.assign({}, defaultConfig), config);
  delete configToWrite._path,

  fs.writeFileSync(filePath, JSON.stringify(configToWrite, null, 2));
}

Object.assign(exports, {
  objectAssignNonnull,
  objectAssignNoclobber,
  loadConfig,
  loadConfigFile,
  saveConfigFile,
  handleCommandError,
});
